package fr.inra.pappso.xtandempipeline.class_msms;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.grouping.HashProtein;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScan;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetPhospho;

public class Match implements Comparable<Match> {
	
	private static final Logger logger = Logger.getLogger(Match.class);
	
	private HashMap<HashSampleScan, Peptide> peptides_match;
	// Par defaut la table est vide, il faut la remplir à postériori

	private ArrayList<Peptide> peptides_order;

	private Protein protein_match;

	private boolean isvalidate;

	public int compareTo(Match p) {
		int k = p.get_valid_peptide_number() - this.get_valid_peptide_number();
		if (k != 0)
			return (k);
		int i = p.peptides_match.size() - this.peptides_match.size();
		if (i != 0)
			return (i);
		return (this.protein_match.compareTo(p
				.get_protein_match()));
	}

	public Match() {
		this.peptides_match = new HashMap<HashSampleScan, Peptide>();
		this.peptides_order = new ArrayList<Peptide>();
		this.protein_match = new Protein();
		this.isvalidate = true;
	}
	
//	@Override
//	protected void finalize() throws Throwable {
//		logger.debug("finalize Match");
//	}

	public void add_peptide_match(Peptide P) {
		// if (!this.peptides_match.containsKey(P.get_sample_scan_cle()))
		// this.peptides_match.put(P.get_sample_scan_cle(), P);
		this.peptides_order.add(P);
	}

	public void IndexAndOrderPeptideListe() throws MSMSException  {
		this.peptides_match.clear();
		Collections.sort(peptides_order);
		for (Peptide pep : peptides_order) {
			if (!this.peptides_match.containsKey(pep.getHashSampleScan())){
				this.peptides_match.put(pep.getHashSampleScan(), pep);
			}else{
				if(this.peptides_match.get(pep.getHashSampleScan()).get_evalue()>pep.get_evalue()){
					throw new MSMSException("OUPS problems of peptide order"+pep.get_sample_scan_sequencepeptide_cle());
				}
			}
		}
	}

	public ArrayList<Peptide> get_all_peptides() {
		return (peptides_order);
	}

	public ArrayList<Peptide> get_valid_peptides() {
		ArrayList<Peptide> peps = new ArrayList<Peptide>();
		for (Peptide pep : peptides_order) {
			if (pep.isValidate())
				peps.add(pep);
		}
		return (peps);
	}

	public void set_protein_match(Protein P) {
		this.protein_match = P;
	}

	public Protein get_protein_match() {
		return (this.protein_match);
	}

	public MsRunSet get_sample_matchs() {
		MsRunSet samples = new MsRunSet();
		for (Peptide pep : peptides_match.values()) {
			samples.add(pep.getMsRun());
		}
		return (samples);
	}

	public int get_spectra_numbers_to(MsRun samp) {
		int i = 0;
		for (Peptide pep : peptides_match.values()) {
			if (pep.getMsRun().equals(samp))
				i++;
		}
		return (i);
	}

	public int get_spectra_numbers() {
		return (this.peptides_match.size());
	}

	public int get_unique_valid_peptide_number_to(MsRun samp) {
		HashSet<String> sequence = new HashSet<String>();
		for (Peptide pep : peptides_match.values()) {
			if (pep.isValidate()) {
				if (pep.getMsRun().equals(samp)) {
					sequence.add(pep.get_sequence());
				}
			}
		}
		return (sequence.size());
	}

	public int get_valid_peptide_number() {
		int i = 0;
		for (Peptide pep : peptides_match.values()) {
			if (pep.isValidate())
				i++;
		}
		return (i);
	}

	public ArrayList<Peptide> get_peptide_order_inclus_redon() {
		Collections.sort(peptides_order);
		return (peptides_order);
	}

	public ArrayList<Peptide> get_peptide_order() {
		ArrayList<Peptide> peptides = new ArrayList<Peptide>(
				peptides_match.values());
		Collections.sort(peptides);
		return (peptides);
	}

	public int get_valid_peptide_number_to(MsRun samp) {
		int i = 0;
		for (Peptide pep : peptides_match.values()) {
			if (pep.getMsRun().equals(samp) && pep.isValidate()) {
				i++;
			}
		}
		return (i);
	}

	public int get_unique_valid_peptide_number() {
		HashSet<String> sequence = new HashSet<String>();
		for (Peptide pep : peptides_match.values()) {
			if (pep.isValidate()) {
				sequence.add(pep.get_sequence());
			}
		}
		return (sequence.size());
	}

	public int get_unique_valid_peptide_number_to_sequencemodifs() {
		HashSet<String> sequence = new HashSet<String>();
		for (Peptide pep : peptides_match.values()) {
			if (pep.isValidate()) {
				sequence.add(pep.get_LI_sequence_modifs());
			}
		}
		return (sequence.size());
	}

	public int get_coverage_to(MsRun samp) {
		//Unknown sequence
		if(this.protein_match.get_sequence().length()==0){
			return 0;
		}
		
		HashMap<Integer, Boolean> seq = new HashMap<Integer, Boolean>();
		for (int i = 0; i < this.protein_match.get_sequence().length(); i++) {
			int i_po = i + 1;
			seq.put(i_po, false);
		}
		for (Peptide pep : peptides_match.values()) {
			if (pep.getMsRun().equals(samp)) {
				for (int j = pep.get_start(); j <= pep.get_stop(); j++) {
					seq.put(j, true);
				}
			}
		}
		int k = 0;
		for (boolean ST : seq.values()) {
			if (ST)
				k++;
		}

		float cov = (float) k
				/ (this.protein_match.get_sequence().length() + 1) * 100;
		int cov_temp = (int) cov;
		return (cov_temp);
	}

	public int get_coverage() {
		//Unknown sequence
		if(this.protein_match.get_sequence().length()==0){
			return 0;
		}
		
		HashMap<Integer, Boolean> seq = this.get_coverage_hashtable();

		int k = 0;
		for (Boolean ST : seq.values()) {
			if (ST)
				k++;
		}

		float cov = (float) k
				/ (this.protein_match.get_sequence().length() + 1) * 100;
		// System.out.println(cov);
		int cov_temp = (int) cov;
		// System.out.println(cov_temp);
		return (cov_temp);
	}

	public HashMap<Integer, Boolean> get_coverage_hashtable() {
		HashMap<Integer, Boolean> seq = new HashMap<Integer, Boolean>();
		if(this.protein_match.get_sequence().length()==0){
			return(seq);
		}
		
		for (int i = 0; i < this.protein_match.get_sequence().length(); i++) {
			int i_po = i + 1;
			seq.put(i_po, false);
		}
		for (Peptide pep : peptides_match.values()) {
			if (pep.isValidate()) {
				for (int j = pep.get_start(); j <= pep.get_stop(); j++) {
					seq.put(j, true);
				}
			}
		}
		return seq;
	}

	public void setIsvalidate(boolean isvalidate) {
		this.isvalidate = isvalidate;
	}

	public boolean isIsvalidate() {
		return isvalidate;
	}

	public HashProtein getHashProtein() throws Exception {
		return this.protein_match.getHashProtein();
	}

	public ArrayList<HashSampleScanSetPhospho> getHashSampleScanSetPhosphoList()
			throws Exception {
		// TODO
		ArrayList<HashSampleScanSetPhospho> phosphoSiteList = new ArrayList<HashSampleScanSetPhospho>();
		// TODO : determine phosphosite positions
		PhosphoIslandSet islands = new PhosphoIslandSet(this);
		for (PhosphoIsland island : islands) {
			phosphoSiteList.add(new HashSampleScanSetPhospho(this, island
					.getPhosphoPositions()));
		}
		return phosphoSiteList;
	}

	public Peptide getBestPeptideToHashSampleScan(HashSampleScan hash) throws MSMSException {
		Peptide pep = this.peptides_match.get(hash);
		if(pep == null){
			logger.error("Hastable of peptide is not indexed");
			this.IndexAndOrderPeptideListe();
		}
		return (this.peptides_match.get(hash));
	}
	
	public ArrayList<Peptide> getAllValidPeptideToHashSampleScan(HashSampleScan hash) {
		ArrayList<Peptide> peps = new ArrayList<Peptide>();
		for(Peptide pep : this.peptides_order){
			if(pep.getHashSampleScan().equals(hash)&&pep.isValidate()){
				peps.add(pep);
			}
		}
		return (peps);
	}
	
	public HashMap<HashSampleScan, Peptide> getHashSampleScanToPeptide(){
		return(peptides_match);
	}
}
