package fr.inra.pappso.xtandempipeline.filter_print;

import java.util.ArrayList;

import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.class_msms.MsRun;
import fr.inra.pappso.xtandempipeline.class_msms.MsRunSet;
import fr.inra.pappso.xtandempipeline.grouping.Group;
import fr.inra.pappso.xtandempipeline.grouping.GroupSet;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetProt;
import fr.inra.pappso.xtandempipeline.grouping.SubGroup;
import fr.inra.pappso.xtandempipeline.sax_parse.msms_output;

public class print_compar extends print_base {

	public print_compar(msms_output out, MsRunSet samp) {
		super(out, samp);
		logger.info("Print Compar file");
	}

	public void print_header() throws Exception {
		String S = "Groupe" + "\t" + "Sub-group" + "\t" + "Description" + "\t" + "Redundancy"+ "\t" + "MW" + "\t" + "Evalue";
		for(MsRun sample : samples.getMsRunList()){
			S = S + "\t" + sample.getSampleName();
		}
		S = S + "\tType\n";
		out.write(S);
	}

	@Override
	public void execute(GroupSet grouping) throws Exception {
		this.print_header();
		for(Group group : grouping.getGroupList()){
			this.print_group_information(group, "Spectra");
		}
		out.write("\n");

		for(Group group : grouping.getGroupList()){
			this.print_group_information(group, "Specific");
		}
		out.write("\n");

		for(Group group : grouping.getGroupList()){
			this.print_group_information(group, "Unique");
		}
		out.write("\n");

		for(Group group : grouping.getGroupList()){
			this.print_group_information(group, "PAI");
		}
		out.write("\n");
	}

	public void print_group_information(Group group, String type)
			throws Exception {
		for(SubGroup sg : group.getSubGroupSet().getSubGroupList()){
			String subgroup = this.get_subgroup_number(group, sg);
			ArrayList<HashSampleScanSetProt> hprotlist = sg.getHashSampleScanSetProtList();
			Match top = hprotlist.get(0).getMatch();
			
			//données des 
			StringBuilder S = new StringBuilder();
			S.append(group.getNumber());
			S.append("\t"+subgroup);
			S.append("\t"+ top.get_protein_match().get_description());
			if (hprotlist.size() > 1)
				S.append("\t" + hprotlist.size()+"*");
			else
				S.append("\t" + "\"-\"");
			S.append("\t" + top.get_protein_match().get_mw());
			S.append("\t" + top.get_protein_match().get_evalue());
			
			//On ecris les données pour chaque sample
			float value = 0;
			for(MsRun sample : samples.getMsRunList()){
				if (type.equals("Spectra"))
					value = top.get_valid_peptide_number_to(sample);

				if (type.equals("Specific"))
					value = group.getNumberOfSpecificPeptideInSubGroupToSample(hprotlist.get(0),sample);
				if (type.equals("Unique")) {
					value = top.get_unique_valid_peptide_number_to(sample);
				}
				if (type.equals("PAI")) {
					value = (float) top.get_spectra_numbers_to(sample)
							/ top.get_protein_match().get_PAI_count();
				}
				if (value > 0)
					S.append("\t" + value);
				else
					S.append("\t");

			}
			S.append("\t" + type + "\n");
			out.write(S.toString());

		}
	}
}
