package fr.inra.pappso.xtandempipeline.filter_print;

import java.util.ArrayList;

import fr.inra.pappso.xtandempipeline.class_msms.MsRun;
import fr.inra.pappso.xtandempipeline.class_msms.MsRunSet;
import fr.inra.pappso.xtandempipeline.grouping.Group;
import fr.inra.pappso.xtandempipeline.grouping.GroupSet;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetPhospho;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetProt;
import fr.inra.pappso.xtandempipeline.grouping.SubGroup;
import fr.inra.pappso.xtandempipeline.sax_parse.msms_output;

public class print_compar_phospho extends print_base {

	public print_compar_phospho(msms_output out, MsRunSet samp) {
		super(out, samp);
		logger.info("Print Compar file in phospho mode");
	}

	public void print_header() throws Exception {
		String S = "Groupe" + "\t" + "Sub-group" + "\t" + "Description" + "\t"
				+ "Redundancy" + "\t" + "Position";
		for (MsRun sample : samples.getMsRunList()) {
			S = S + "\t" + sample.getSampleName();
		}
		S = S + "\tType\n";
		out.write(S);
	}
	
	@Override
	public void execute(GroupSet grouping) throws Exception {
		this.print_header();
		for(Group group : grouping.getGroupList()){
			this.print_group_information(group, "Spectra");
		}
		out.write("\n");

//		for(Group group : grouping.getGroupList()){
//			this.print_group_information(group, "Specific");
//		}
//		out.write("\n");

		for(Group group : grouping.getGroupList()){
			this.print_group_information(group, "Unique");
		}
		out.write("\n");
//
//		for(Group group : grouping.getGroupList()){
//			this.print_group_information(group, "PAI");
//		}
//		out.write("\n");
	}

	public void print_group_information(Group group, String type)
			throws Exception {
		for (SubGroup sg : group.getSubGroupSet().getSubGroupList()) {
			String subgroup = this.get_subgroup_number(group, sg);
			ArrayList<HashSampleScanSetProt> hprotlist = sg
					.getHashSampleScanSetProtList();
			
			HashSampleScanSetPhospho phospho = (HashSampleScanSetPhospho) hprotlist.get(0);
			
			// données des
			StringBuilder S = new StringBuilder();
			S.append(group.getNumber());
			S.append("\t" + subgroup);
			S.append("\t" + phospho.getMatch().get_protein_match().get_description());
			if (hprotlist.size() > 1)
				S.append("\t" + hprotlist.size() + "*");
			else
				S.append("\t" + "\"-\"");
			
			S.append("\t" + phospho.getPhosphoPositionToPrint());

			// On ecris les données pour chaque sample
			float value = 0;
			for (MsRun sample : samples.getMsRunList()) {
				if (type.equals("Spectra"))
					value = phospho.getTotalHashSampleScantoMsRun(sample);
				
				if (type.equals("Unique")) {
					value = phospho.getUniqueHashPeptideSampleScantoMsRun(sample);
				}
				if (value > 0)
					S.append("\t" + value);
				else
					S.append("\t");

			}
			S.append("\t" + type + "\n");
			out.write(S.toString());

		}
	}

}
