package fr.inra.pappso.xtandempipeline.filter_print;

import java.util.ArrayList;

import fr.inra.pappso.libodsstream.OdsDocWriter;
import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.class_msms.MsRun;
import fr.inra.pappso.xtandempipeline.class_msms.MsRunSet;
import fr.inra.pappso.xtandempipeline.grouping.Group;
import fr.inra.pappso.xtandempipeline.grouping.GroupSet;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetProt;
import fr.inra.pappso.xtandempipeline.grouping.SubGroup;

public class print_ods_compar extends printOdsBase {

	public print_ods_compar(OdsDocWriter odsTable, MsRunSet samp) {
		super(odsTable, samp);
		// super(out, samp);
		logger.info("Print Compar file");
	}

	public void print_header() throws Exception {
		odsTable.writeLine();
		// out.write(samp.getSampleName() + "\n");
		ArrayList<String> headStrList = new ArrayList<String>();
		headStrList.add("Group");
		headStrList.add("Sub-group");
		headStrList.add("Description");
		headStrList.add("Redundancy");
		headStrList.add("Coverage");
		headStrList.add("MW");
		headStrList.add("Evalue");
		for (MsRun sample : samples.getMsRunList()) {
			headStrList.add(sample.getSampleName());
		}

		headStrList.add("Type");

		for (String title : headStrList) {
			odsTable.writeCell(title);
		}
	}

	@Override
	public void execute(GroupSet grouping) throws Exception {
		odsTable.writeSheet("compar_Spectra");
		this.print_header();
		for (Group group : grouping.getGroupList()) {
			this.print_group_information(group, "Spectra");
		}
		odsTable.writeLine();
		// out.write("\n");
		odsTable.writeSheet("compar_Specific");
		this.print_header();
		for (Group group : grouping.getGroupList()) {
			this.print_group_information(group, "Specific");
		}
		odsTable.writeLine();
		// out.write("\n");
		odsTable.writeSheet("compar_Unique");
		this.print_header();
		for (Group group : grouping.getGroupList()) {
			this.print_group_information(group, "Unique");
		}
		odsTable.writeLine();
		// out.write("\n");
		odsTable.writeSheet("compar_PAI");
		this.print_header();
		for (Group group : grouping.getGroupList()) {
			this.print_group_information(group, "PAI");
		}
		odsTable.writeLine();
		// .write("\n");
	}

	public void print_group_information(Group group, String type)
			throws Exception {
		logger.debug("printing group "+ group.getNumber());
		for (SubGroup sg : group.getSubGroupSet().getSubGroupList()) {
			logger.debug("printing subgroup "+ sg.getNumber());
			odsTable.writeLine();
			String subgroup = this.get_subgroup_number(group, sg);
			ArrayList<HashSampleScanSetProt> hprotlist = sg
					.getHashSampleScanSetProtList();
			Match top = hprotlist.get(0).getMatch();

			// données des
			// StringBuilder S = new StringBuilder();
			odsTable.writeCell("" + group.getNumber());
			// S.append(group.getNumber());
			odsTable.writeCell("" + subgroup);
			// S.append("\t"+subgroup);
			odsTable.writeCell(top.get_protein_match()
					.get_description());
			// S.append("\t"+ top.get_protein_match().get_description());
			if (hprotlist.size() > 1) {
				odsTable.writeCell(hprotlist.size() + "*");
				// S.append("\t" + hprotlist.size()+"*");
			} else {
				odsTable.writeCell("-");
				// S.append("\t" + "\"-\"");
			}
			odsTable.writeCell(new Double(top.get_coverage()));
			odsTable.writeCell(new Double(top.get_protein_match()
					.get_mw()));
			// S.append("\t" + top.get_protein_match().get_mw());
			odsTable.writeCell(new Double(top.get_protein_match()
					.get_evalue()));
			// S.append("\t" + top.get_protein_match().get_evalue());

			// On ecris les données pour chaque sample
			float value = 0;
			for (MsRun sample : samples.getMsRunList()) {
				if (type.equals("Spectra"))
					value = top.get_valid_peptide_number_to(sample);

				if (type.equals("Specific"))
					value = group.getNumberOfSpecificPeptideInSubGroupToSample(
							hprotlist.get(0), sample);
				if (type.equals("Unique")) {
					value = top.get_unique_valid_peptide_number_to(sample);
				}
				if (type.equals("PAI")) {
					value = (float) top.get_spectra_numbers_to(sample)
							/ top.get_protein_match().get_PAI_count();
				}
				if (value > 0) {
					odsTable.writeCell(new Double(value));
					// S.append("\t" + value);
				} else {
					odsTable.writeCell("");

					// S.append("\t");
				}
			}
			odsTable.writeCell(type);
			// S.append("\t" + type + "\n");
			// out.write(S.toString());

		}
	}
}
