/*******************************************************************************
 * Copyright (c) 2012 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 * 
 * This file is part of XTandemPipeline.
 * 
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Contributors:
 *     Benoit Valot <Benoit.Valot@moulon.inra.fr>
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr>
 ******************************************************************************/

package fr.inra.pappso.xtandempipeline.sax_parse;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import org.apache.log4j.Logger;

import fr.inra.moulon.proticport.model.exceptions.ProticPortException;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.class_msms.Modifs;
import fr.inra.pappso.xtandempipeline.class_msms.ModifsList;
import fr.inra.pappso.xtandempipeline.class_msms.MsRun;
import fr.inra.pappso.xtandempipeline.class_msms.Peptide;
import fr.inra.pappso.xtandempipeline.class_msms.Protein;
import fr.inra.pappso.xtandempipeline.grouping.Group;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScan;
import fr.inra.pappso.xtandempipeline.grouping.SubGroup;
import fr.inra.pappso.xtandempipeline.grouping.SubGroupSet;

public class MassChroQmlWriter {
	private static final Logger logger = Logger
			.getLogger(MassChroQmlWriter.class);

	static String namespaceURI = "http://sashimi.sourceforge.net/schema_revision/mzXML_2.0";
	static String xmlnsxsi = "http://www.w3.org/2001/XMLSchema-instance";
	static String xsischemaLocation = "http://sashimi.sourceforge.net/schema_revision/mzXML_2.0 http://sashimi.sourceforge.net/schema_revision/mzXML_2.0/mzXML_idx_2.0.xsd";

	protected XMLStreamWriter writer;

	private Hashtable<MsRun, String> sample2xmlId;
	private String groupe_id;

	MassChroQmlWriter(XMLStreamWriter writer) throws ProticPortException {
		this.sample2xmlId = new Hashtable<MsRun, String>();
		this.groupe_id = "G1";
		this.writer = writer;

	}

	public void write(ModifsList isotopic_mods, Identification ident)
			throws XMLStreamException, ProticPortException {
		logger.debug("write begin");
		long duracel = System.currentTimeMillis();
		//writer.setPrefix("xsi", xmlnsxsi);
		//writer.setDefaultNamespace(namespaceURI);
		writer.writeStartElement("masschroq");
		//writer.writeNamespace("xsi", xmlnsxsi);
		//writer.writeDefaultNamespace(namespaceURI);

		//writer.writeAttribute(xmlnsxsi, "schemaLocation", xsischemaLocation);

		this.writeRawData(ident);
		this.writeGroups(ident);
		this.writeProteins(ident);
		this.writePeptides(ident);
		this.writeIsotopes(isotopic_mods);

		// ajout des methodes
		this.writeAlignement();

		// quantification_methods
		this.writeMethods();

		// ajout des instructions de quanti
		this.writeQuantification(isotopic_mods);
		// sauvegarde

		logger.info("Duration creating MassChroQml xml document: "
				+ (System.currentTimeMillis() - duracel) + " ms");
		// masschroq
		writer.writeEndElement();

	}

	private void writeQuantification(ModifsList mods) {
		logger.debug("write begin");
		try {
			writer.writeStartElement("quantification");

			// Element result = doc.createElement("quantification_results");
			writer.writeStartElement("quantification_results");
			// Element output = doc.createElement("quantification_result");
			writer.writeEmptyElement("quantification_result");
			writer.writeAttribute("output_file", "XIC_result");
			writer.writeAttribute("format", "tsv");
			writer.writeEndElement();// quantification_results

			// Element trace = doc.createElement("quantification_traces");
			writer.writeStartElement("quantification_traces");
			// Element peptides_traces = doc.createElement("peptide_traces");
			writer.writeEmptyElement("peptide_traces");
			writer.writeAttribute("peptide_ids", "pep0 pep1");
			writer.writeAttribute("output_dir", "peplist_xics_traces/");
			writer.writeAttribute("format", "tsv");
			writer.writeEndElement();// quantification_traces

			// Element quantify = doc.createElement("quantify");
			writer.writeStartElement("quantify");
			writer.writeAttribute("id", "q1");
			writer.writeAttribute("withingroup", this.groupe_id);
			writer.writeAttribute("quantification_method_id", "quant1");
			// Element peptides_in_peptide_list = doc
			// .createElement("peptides_in_peptide_list");
			writer.writeEmptyElement("peptides_in_peptide_list");
			writer.writeAttribute("mode", "real_or_mean");

			// ajout des labels isotopics dans la methode
			if (mods.size() > 0) {
				String label = "";
				for (int z = 0; z < mods.size(); z++) {
					label = label + " " + "iso" + (z + 1);
				}
				writer.writeAttribute("isotope_label_refs", label.trim());
			}

			writer.writeEndElement();// quantify
			writer.writeEndElement();// quantification
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
		logger.debug("write end");

	}

	private void writeMethods() {
		logger.debug("write begin");
		try {
			writer.writeStartElement("quantification_methods");
			// Element quantification_method =
			// doc.createElement("quantification_method");
			writer.writeStartElement("quantification_method");
			writer.writeAttribute("id", "quant1");
			// quantification_methods.appendChild(quantification_method);

			// Element xic_extraction = doc.createElement("xic_extraction");
			writer.writeStartElement("xic_extraction");
			writer.writeAttribute("xic_type", "max");
			// quantification_method.appendChild(xic_extraction);
			writer.writeComment("max : XIC on BasePeak; sum : XIC on TIC");

			// selection
			// Element ppm_range = doc.createElement("ppm_range");
			writer.writeEmptyElement("ppm_range");
			writer.writeAttribute("min", "10");
			writer.writeAttribute("max", "10");
			writer.writeComment("For XIC extraction on Da use: mz_range");

			writer.writeEndElement();// xic_extraction

			// xic_filter
			// Element xic_filters = doc.createElement("xic_filters");
			writer.writeStartElement("xic_filters");

			// Element anti_spike = doc.createElement("anti_spike");
			writer.writeEmptyElement("anti_spike");
			writer.writeAttribute("half", "5");

			// Element background = doc.createElement("background");
			writer.writeEmptyElement("background");
			writer.writeAttribute("half_mediane", "5");
			writer.writeAttribute("half_min_max", "20");

			writer.writeEndElement();// xic_filters

			// detection michel trop bien
			// Element peak_detection = doc.createElement("peak_detection");
			writer.writeStartElement("peak_detection");

			// Element detection = doc.createElement("detection_zivy");
			writer.writeStartElement("detection_zivy");
			// Element smoothing = doc.createElement("mean_filter_half_edge");
			writer.writeStartElement("mean_filter_half_edge");
			writer.writeCharacters("1");
			writer.writeEndElement();// mean_filter_half_edge

			// Element minmax = doc.createElement("minmax_half_edge");
			writer.writeStartElement("minmax_half_edge");
			writer.writeCharacters("3");
			writer.writeEndElement();// minmax_half_edge

			// Element maxmin = doc.createElement("maxmin_half_edge");
			writer.writeStartElement("maxmin_half_edge");
			writer.writeCharacters("2");
			writer.writeEndElement();// maxmin_half_edge

			// Element tic_start =
			// doc.createElement("detection_threshold_on_max");
			writer.writeStartElement("detection_threshold_on_max");
			writer.writeCharacters("5000");
			writer.writeEndElement();// detection_threshold_on_max

			// Element tic_stop =
			// doc.createElement("detection_threshold_on_min");
			writer.writeStartElement("detection_threshold_on_min");
			writer.writeCharacters("3000");
			writer.writeEndElement();// detection_threshold_on_min

			writer.writeEndElement();// detection_zivy
			writer.writeEndElement();// peak_detection
			
			writer.writeEndElement();// quantification_method
			writer.writeEndElement();// quantification_methods
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
		logger.debug("write end");

	}

	private void writeAlignement() {
		logger.debug("write begin");
		try {
			writer.writeStartElement("alignments");
			// methods
			// Element alignment_methods =
			// doc.createElement("alignment_methods");
			writer.writeStartElement("alignment_methods");

			// Element alignment_method = doc.createElement("alignment_method");
			writer.writeStartElement("alignment_method");
			// alignment_methods.appendChild(alignment_method);

			writer.writeAttribute("id", "ms2_1");
			// Element ms2 = doc.createElement("ms2");
			writer.writeStartElement("ms2");

			// Element tendency = doc.createElement("ms2_tendency_halfwindow");
			writer.writeStartElement("ms2_tendency_halfwindow");
			writer.writeCharacters("10");
			writer.writeEndElement();

			// Element smoothing =
			// doc.createElement("ms2_smoothing_halfwindow");
			writer.writeStartElement("ms2_smoothing_halfwindow");
			writer.writeCharacters("15");
			writer.writeEndElement();

			// Element smoothing2 =
			// doc.createElement("ms1_smoothing_halfwindow");
			writer.writeStartElement("ms1_smoothing_halfwindow");
			writer.writeCharacters("0");
			writer.writeEndElement();

			// alignment_method.appendChild(ms2);
			writer.writeEndElement();// ms2
			writer.writeEndElement();// alignment_method
			writer.writeEndElement();// alignment_methods

			// groupe d'alignement
			// Element align = doc.createElement("align");
			writer.writeEmptyElement("align");
			writer.writeAttribute("group_id", this.groupe_id);
			writer.writeAttribute("method_id", "ms2_1");
			writer.writeAttribute("reference_data_id", "samp0");

			writer.writeEndElement();// alignments
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
		logger.debug("write end");

	}

	private void writeRawData(Identification ident) {
		logger.debug("write begin");
		try {
			writer.writeStartElement("rawdata");
			for (MsRun sample : ident.get_samples().getMsRunList()) {
				String id_samp = "samp" + this.sample2xmlId.size();
				// ajout dans les hashtable
				sample2xmlId.put(sample, id_samp);

				// balise rawdata
				// Element data_file = doc.createElement("data_file");
				writer.writeEmptyElement("data_file");
				// data_file.setAttribute("id", id_samp);
				writer.writeAttribute("id", id_samp);
				writer.writeAttribute("format", "mzxml");
				writer.writeAttribute("path", "" + sample.getSampleName()
						+ ".mzXML");
				writer.writeAttribute("type", "profile");
			}

			writer.writeEndElement();
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
		logger.debug("write end");

	}

	private void writeGroups(Identification ident) {
		logger.debug("write begin");
		try {
			writer.writeStartElement("groups");

			// //balise groups
			// Element group = doc.createElement("group");
			writer.writeEmptyElement("group");
			writer.writeAttribute("id", this.groupe_id);
			String raw_list = "";
			for (MsRun sample : ident.get_samples().getMsRunList()) {
				raw_list = raw_list + " " + sample2xmlId.get(sample);
			}
			writer.writeAttribute("data_ids", raw_list.trim());
			writer.writeEndElement();
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
		logger.debug("write end");

	}

	private void writeProteins(Identification ident) {
		// protein_list
		logger.debug("write begin");
		try {
			writer.writeStartElement("protein_list");

			for (Group group : ident.getGrouping().getGroupList()) {
				for (SubGroup sg : group.getSubGroupSet().getSubGroupList()) {
					Protein top = sg.getHashSampleScanSetProtList().get(0)
							.getMatch().get_protein_match();

					// balise proteines
					String prot_id = this.getSubGroupGroupNumber(group, sg);

					// Element protein = doc.createElement("protein");
					writer.writeEmptyElement("protein");
					writer.writeAttribute("id", prot_id);
					writer.writeAttribute("desc", top.get_description());
				}
			}

			writer.writeEndElement();
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
		logger.debug("write end");
	}

	private void writePeptides(Identification ident) {
		logger.debug("write begin");
		/*
		 * <peptide_list> <peptide id="pep0" mh="1463.626" mods="114.08"
		 * prot_ids="P1.1" seq="TCVADESHAGCEK"> <observed_in data="samp0"
		 * scan="655" z="2" /> <observed_in data="samp1" scan="798" z="2" />
		 * </peptide> <peptide id="pep1" mh="1103.461" mods="57.04"
		 * prot_ids="P1.1" seq="ADESHAGCEK"> <observed_in data="samp3"
		 * scan="663" z="2" /> </peptide> </peptide_list>
		 */
		try {
			writer.writeStartElement("peptide_list");
			Hashtable<String, HashSet<String>> seqMod2protIds;
			Hashtable<String, String> xmlPeptideId2seqMod;
			Hashtable<String, ArrayList<Peptide>> seqMod2PeptideList;
			ArrayList<String> peptideId;

			int i = 0;

			for (Group group : ident.getGrouping().getGroupList()) {
				seqMod2protIds = new Hashtable<String, HashSet<String>>();
				xmlPeptideId2seqMod = new Hashtable<String, String>();
				seqMod2PeptideList = new Hashtable<String, ArrayList<Peptide>>();
				peptideId = new ArrayList<String>();

				for (HashSampleScan hash : group.getHashSampleScanSet()
						.getHashSampleScanList()) {

					SubGroupSet set = group.getSubGroupContaining(hash);
					ArrayList<SubGroup> sgList = set.getSubGroupList();
					Match top = sgList.get(0).getHashSampleScanSetProtList()
							.get(0).getMatch();
					Peptide pep = top.getBestPeptideToHashSampleScan(hash);

					String seqMod = pep.get_LI_sequenceModifsWithoutMs3();
					if (seqMod2protIds.containsKey(seqMod)) {
					} else {
						//Add new peptide id
						String pepid = "pep" + i++;
						xmlPeptideId2seqMod.put(pepid,seqMod);
						peptideId.add(pepid);
						/*
						 * <peptide id="pep0" mh="1463.626" mods="114.08"
						 * prot_ids="P1.1" seq="TCVADESHAGCEK">
						 */
						// table temporaire pour prot_id
						HashSet<String> temp = new HashSet<String>();
						seqMod2protIds.put(seqMod, temp);

						seqMod2PeptideList
								.put(seqMod, new ArrayList<Peptide>());
					}

					seqMod2PeptideList.get(seqMod).add(pep);

					for (SubGroup sgTemp : sgList) {
						HashSet<String> temp = seqMod2protIds.get(seqMod);
						temp.add(this.getSubGroupGroupNumber(group, sgTemp));
					}
				}// end loop on each scan for one group
				
				//iterated current pepid to write buffer peptide element
				for (String pepid : peptideId) {
					String seqMod = xmlPeptideId2seqMod.get(pepid);
					ArrayList<Peptide> peptideList = seqMod2PeptideList
							.get(seqMod);
					writer.writeStartElement("peptide");
					writer.writeAttribute("id", pepid);
					Peptide peptide = peptideList.get(0);
					// this.seq_to_element.put(seqMod, peptide);
					writer.writeAttribute("mh",
							"" + peptide.get_mh_theo_without_isotope(true));
					writer.writeAttribute("seq", peptide.get_sequence());
					writer.writeAttribute("mods",
							"" + peptide.get_Modifs_formater_without_isotope());

					HashSet<String> temp = seqMod2protIds.get(seqMod);
					Iterator<String> prots = temp.iterator();
					String ids = prots.next();
					while (prots.hasNext())
						ids = ids + " " + prots.next();
					writer.writeAttribute("prot_ids", ids.trim());

					for (Peptide peptideScan : peptideList) {

						// Element observed = doc.createElement("observed_in");
						writer.writeEmptyElement("observed_in");
						// eda choisir mode export
						// observed.setAttribute("rt", "" +
						// peps[j].get_RT_in_sec());
						writer.writeAttribute("data",
								this.sample2xmlId.get(peptideScan.getMsRun()));
						writer.writeAttribute("scan",
								"" + peptideScan.get_scan());
						writer.writeAttribute("z",
								"" + peptideScan.get_charge());
					}

					writer.writeEndElement();
				}
			}
			writer.writeEndElement();
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
		logger.debug("write end");
	}

	private void writeIsotopes(ModifsList mods) {
		logger.debug("write begin");
		/*
		 * <peptide_list> <peptide id="pep0" mh="1463.626" mods="114.08"
		 * prot_ids="P1.1" seq="TCVADESHAGCEK"> <observed_in data="samp0"
		 * scan="655" z="2" /> <observed_in data="samp1" scan="798" z="2" />
		 * </peptide> <peptide id="pep1" mh="1103.461" mods="57.04"
		 * prot_ids="P1.1" seq="ADESHAGCEK"> <observed_in data="samp3"
		 * scan="663" z="2" /> </peptide> </peptide_list>
		 */
		try {
			Modifs[] mds = mods.get_modifs_order();
			if (mds.length > 0) {
				writer.writeStartElement("isotope_label_list");

				for (int i = 0; i < mds.length; i++) {
					// Element isotope_label =
					// doc.createElement("isotope_label");
					writer.writeStartElement("isotope_label");
					writer.writeAttribute("id", "iso" + (i + 1));
					// Element mod = doc.createElement("mod");
					writer.writeEmptyElement("mod");
					writer.writeAttribute("value", "" + mds[i].get_modvalue());
					writer.writeAttribute("at", "Nter,Cter,A,K,L...");

					// isotopes.appendChild(isotope_label);
					writer.writeEndElement();
				}

				writer.writeEndElement();
			}
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
		logger.debug("write end");
	}

	private String getSubGroupGroupNumber(Group group, SubGroup sg) {
		String prot_id = "P" + group.getNumber() + ".";
		if (sg.getNumber() < 10)
			prot_id += "0" + sg.getNumber();
		else
			prot_id += sg.getNumber();
		return (prot_id);
	}

}
