package fr.inra.pappso.xtandempipeline.sax_parse;

import org.apache.log4j.Logger;
import org.w3c.dom.*;

import javax.xml.parsers.*;
import java.io.*;
import java.util.ArrayList;
import java.util.Hashtable;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.*;
import fr.inra.pappso.xtandempipeline.grouping.Group;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetProt;
import fr.inra.pappso.xtandempipeline.grouping.SubGroup;

public class proticdbml {
	private static final Logger logger = Logger.getLogger(proticdbml.class);
	
	private Identification ident;

	private File proticfile;

	private Document doc;

	private Element racine;

	private Element sequences;

	private Element samples;

	private Element msruns;

	private Element identificationruns;

	private Element identificationrun;

	private Hashtable<String, String> prot_to_id;

	private Hashtable<String, String> pep_to_id;

	private Hashtable<String, String> sample_to_id;

	private Hashtable<String, String> msrun_to_id;

	public proticdbml(File F) throws Exception {
		logger.info("Save result in PROTICdbML");
		this.proticfile = F;
		this.prot_to_id = new Hashtable<String, String>();
		this.pep_to_id = new Hashtable<String, String>();
		this.sample_to_id = new Hashtable<String, String>();
		this.msrun_to_id = new Hashtable<String, String>();

		try {
			// creation et ouverture d'un DOM
			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder parser = factory.newDocumentBuilder();
			this.doc = parser.newDocument();
			this.racine = doc.createElement("PROTICdb");
			doc.appendChild(this.racine);
			initialise();
			write();

		} catch (Exception e) {
			throw (e);
		}
	}

	private void initialise() {
		// Header du fichier
		this.racine.setAttribute("xmlns:xsi",
				"http://www.w3.org/2001/XMLSchema-instance");
		this.racine.setAttribute("version", "1.0");
		this.racine
				.setAttribute(
						"xsi:noNamespaceSchemaLocation",
						"http://uln.moulon.inra.fr/~langella/proticdev/doc/xml_schema/proticdbml/proticdbml.xsd");
		this.racine.setAttribute("type", "MSidentificationResults");
		// Ajout des méthodes
		Element identMeth = doc.createElement("identMeth");
		identMeth.setAttribute("name", "");
		identMeth.setAttribute("id", "m1");
		racine.appendChild(identMeth);

		// Ajout des base de données
		Element customDb = doc.createElement("customDb");
		customDb.setAttribute("name", "");
		customDb.setAttribute("id", "customdb0");
		racine.appendChild(customDb);

		// ajout des elements listes
		this.sequences = doc.createElement("sequences");
		racine.appendChild(sequences);
		Element project = doc.createElement("project");
		project.setAttribute("name", "");
		project.setAttribute("id", "p1");
		racine.appendChild(project);

		this.samples = doc.createElement("samples");
		project.appendChild(samples);
		this.msruns = doc.createElement("msRuns");
		project.appendChild(msruns);
		this.identificationruns = doc.createElement("identificationRuns");
		project.appendChild(identificationruns);
	}

	public void add_identification(Identification id) throws Exception {
		try {
			this.ident = id;
			// ajout d'une mnip d'identification
			this.add_identificationrun();
			// sauvegarde
			this.write();

			// ajout des séquences
			this.add_sequences();
			// sauvegarde
			this.write();

			// ajout des echantillons_msrun
			this.add_samples_msruns();
			// sauvegarde
			this.write();

			// ajout des petides
			this.add_peptidehits();
			// sauvegarde
			this.write();

			// ajout des mathces
			this.add_match();

			// sauvegarde
			this.write();
		} catch (Exception e) {
			throw (e);
		}

	}

	private void add_samples_msruns() {
		for (MsRun samp : ident.get_samples()) {
			String id_samp = "samp" + sample_to_id.size();
			String id_msrun = "msr" + msrun_to_id.size();
			String name = samp.getSampleName();
			name = name.replaceAll(".xml", "");
			// balise sample
			Element sample = doc.createElement("sample");
			sample.setAttribute("name", name);
			sample.setAttribute("id", id_samp);
			samples.appendChild(sample);
			Element description = doc.createElement("description");
			sample.appendChild(description);
			sample_to_id.put(samp.getSampleName(), id_samp);

			// balise MsRun
			Element msRun = doc.createElement("msRun");
			msRun.setAttribute("sample_id", id_samp);
			msRun.setAttribute("id", id_msrun);
			msruns.appendChild(msRun);

			Element description2 = doc.createElement("description");
			msRun.appendChild(description2);
			Element admin = doc.createElement("admin");
			description2.appendChild(admin);
			Element contact = doc.createElement("contact");
			admin.appendChild(contact);
			Element email = doc.createElement("email");
			email.setTextContent("valot@moulon.inra.fr");
			contact.appendChild(email);
			Element nameb = doc.createElement("name");
			nameb.setTextContent("Valot Benoit");
			contact.appendChild(nameb);
			Element sourceFile = doc.createElement("sourceFile");
			admin.appendChild(sourceFile);
			Element nameOfFile = doc.createElement("nameOfFile");
			nameOfFile.setTextContent(name + ".RAW");
			sourceFile.appendChild(nameOfFile);
			Element sampleName = doc.createElement("sampleName");
			sampleName.setTextContent(name);
			admin.appendChild(sampleName);

			// Element spectrumList = doc.createElement("spectrumList");
			// msRun.appendChild(spectrumList);

			msrun_to_id.put(samp.getSampleName(), id_msrun);
		}

	}

	private void add_sequences() {
		for(Group group : ident.getGrouping().getGroupList()){
			for(SubGroup sg : group.getSubGroupSet().getSubGroupList()){
				for(HashSampleScanSetProt hashProt : sg.getHashSampleScanSetProtList()){				
					Protein prot = hashProt.getMatch().get_protein_match();
					if (prot_to_id.containsKey(prot.get_description()) == false) {
						Element sequence = doc.createElement("sequence");
						String id = "seq" + prot_to_id.size();
						if (prot.get_description().length() > 60)
							sequence.setAttribute("display_id", prot
									.get_description().substring(0, 59));
						else
							sequence.setAttribute(
									"display_id",
									prot.get_description().substring(0,
											prot.get_description().length()));
						sequence.setAttribute("id", id);
						sequences.appendChild(sequence);
						// <dbxref key="AT5G16390"
						// dbname="AGI_LocusCode"></dbxref>
						if (prot.get_dbxref_type().equals("no") == false) {
							Element dbxref = doc.createElement("dbxref");
							dbxref.setAttribute("key",
									prot.get_dbxref_accession());
							dbxref.setAttribute("dbname",
									prot.get_dbxref_type());
							sequence.appendChild(dbxref);
						}
						Element description = doc.createElement("description");
						description.setTextContent(prot.get_description());
						sequence.appendChild(description);
						Element seq = doc.createElement("seq");
						seq.setTextContent(prot.get_sequence());
						sequence.appendChild(seq);

						prot_to_id.put(prot.get_description(), id);
					}
				}
			}
		}
	}

	private void add_identificationrun() {
		this.identificationrun = doc.createElement("identificationRun");
		identificationrun.setAttribute("ident_method_id", "m1");
		identificationrun.setAttribute("customdb_id", "customdb0");
		this.identificationruns.appendChild(identificationrun);

		Element description = doc.createElement("description");
		identificationrun.appendChild(description);
		Element admin = doc.createElement("admin");
		description.appendChild(admin);
		Element contact = doc.createElement("contact");
		admin.appendChild(contact);
		Element email = doc.createElement("email");
		email.setTextContent("valot@moulon.inra.fr");
		contact.appendChild(email);
		Element nameb = doc.createElement("name");
		nameb.setTextContent("Valot Benoit");
		contact.appendChild(nameb);
		Element dataProcessing = doc.createElement("dataProcessing");
		description.appendChild(dataProcessing);
		Element software = doc.createElement("software");
		dataProcessing.appendChild(software);
		Element name = doc.createElement("name");
		name.setAttribute("acc", "PROTICdbO:0000283");
		name.setTextContent("Xtandem");
		software.appendChild(name);
		Element version = doc.createElement("version");
		version.setTextContent("2008.02.01.3");
		software.appendChild(version);

	}

	private void add_peptidehits() {
		Element peptideHits = doc.createElement("peptideHits");
		identificationrun.appendChild(peptideHits);
		for(Group group : ident.getGrouping().getGroupList()){
			for(SubGroup sg : group.getSubGroupSet().getSubGroupList()){
				for(HashSampleScanSetProt hashProt : sg.getHashSampleScanSetProtList()){
					for(Peptide pep : hashProt.getMatch().get_peptide_order_inclus_redon()){
						if (pep_to_id.containsKey(pep
								.get_sample_scan_sequencepeptide_cle()) == false) {

							peptideHits.appendChild(this
									.add_peptide_hit(pep));

						}
					}
				}
			}
		}
	}

	private Element add_peptide_hit(Peptide pep) {
		String pep_hit_id = "pep" + pep_to_id.size();
		// param par default
		Element peptideHit = doc.createElement("peptideHit");
		peptideHit.setAttribute("calc_mr", ""
				+ (pep.get_mhplus_theo() - 1.00794));
		double exp_mz = (pep.get_mhplus_obser() + (1.00794 * (pep.get_charge() - 1)))
				/ pep.get_charge();
		peptideHit.setAttribute("exp_mz", "" + exp_mz);
		peptideHit.setAttribute("delta", "" + pep.get_deltamass());
		peptideHit.setAttribute("exp_mr", ""
				+ (pep.get_mhplus_obser() - 1.00794));
		peptideHit.setAttribute("acq_number", "" + pep.get_scan());
		peptideHit.setAttribute("ms_run_id",
				"" + msrun_to_id.get(pep.get_sample()));
		peptideHit.setAttribute("id", pep_hit_id);
		peptideHit.setAttribute("exp_z", "" + pep.get_charge());

		// cvparam specifique xtandem
		Element cvParam1 = doc.createElement("cvParam");
		cvParam1.setAttribute("value", "" + pep.get_evalue());
		cvParam1.setAttribute("name", "xtandem peptide evalue");
		cvParam1.setAttribute("cvLabel", "PROTICdbO");
		cvParam1.setAttribute("accession", "PROTICdbO:0000287");
		peptideHit.appendChild(cvParam1);

		Element cvParam2 = doc.createElement("cvParam");
		cvParam2.setAttribute("value", "" + pep.get_hypercorr());
		cvParam2.setAttribute("name", "xtandem peptide hyperscore");
		cvParam2.setAttribute("cvLabel", "PROTICdbO");
		cvParam2.setAttribute("accession", "PROTICdbO:0000288");
		peptideHit.appendChild(cvParam2);

		Element cvParam3 = doc.createElement("cvParam");
		cvParam3.setAttribute("value", "" + pep.get_RT());
		cvParam3.setAttribute("name", "xtandem peptide retention time");
		cvParam3.setAttribute("cvLabel", "PROTICdbO");
		cvParam3.setAttribute("accession", "PROTICdbO:0000289");
		peptideHit.appendChild(cvParam3);

		// Element cvParam4 = doc.createElement("cvParam");
		// cvParam4.setAttribute("value", "" + pep.get_post());
		// cvParam4.setAttribute("name", "xtandem peptide post");
		// cvParam4.setAttribute("cvLabel", "PROTICdbO");
		// cvParam4.setAttribute("accession", "PROTICdbO:00000XX");
		// peptideHit.appendChild(cvParam4);
		//
		// Element cvParam5 = doc.createElement("cvParam");
		// cvParam5.setAttribute("value", "" + pep.get_pre());
		// cvParam5.setAttribute("name", "xtandem peptide pre");
		// cvParam5.setAttribute("cvLabel", "PROTICdbO");
		// cvParam5.setAttribute("accession", "PROTICdbO:00000XX");
		// peptideHit.appendChild(cvParam5);

		// sequences avec les modifs
		Element pepSeq = doc.createElement("pepSeq");
		peptideHit.appendChild(pepSeq);
		Element peptide = doc.createElement("peptide");
		peptide.setTextContent("" + pep.get_sequence());
		pepSeq.appendChild(peptide);

		if (pep.get_Modifs().size() != 0)
			pepSeq.appendChild(this.add_modifs(pep.get_Modifs()));

		pep_to_id.put(pep.get_sample_scan_sequencepeptide_cle(), pep_hit_id);
		return (peptideHit);
	}

	private Element add_modifs(ArrayList<Modifs> mods) {
		Element ptms = doc.createElement("ptms");
		for (Modifs mod : mods) {
			Element ptm = doc.createElement("ptm");
			ptm.setAttribute("diff_mono", "" + mod.get_modvalue());
			ptm.setAttribute("position", "" + mod.get_posi());
			ptm.setAttribute("aa", "" + mod.get_AA());
			ptms.appendChild(ptm);
			Element obo_ref = this.get_obo_modif_ref(mod.get_modvalue());
			if (obo_ref.getAttribute("name").equals("no") == false)
				ptm.appendChild(obo_ref);

		}

		return (ptms);
	}

	private Element get_obo_modif_ref(float F) {
		// <cvParam name="methionine oxidation with neutral loss of 80 Da"
		// cvLabel="PSI-MOD" accession="MOD:00768"></cvParam>
		Element obo_ref = doc.createElement("cvParam");
		boolean have_amodif = false;
		if (((F - 15.99) > -0.01) && ((F - 15.99) < 0.01)) {
			obo_ref.setAttribute("name", "oxidation to L-methionine sulfoxide");
			obo_ref.setAttribute("cvLabel", "PSI-MOD");
			obo_ref.setAttribute("accession", "MOD:00719");
			have_amodif = true;
			// oxidation to L-methionine sulfoxide [MOD:00719] 15.994915
		}
		if (((F - 57.02) > -0.03) && ((F - 57.02) < 0.03)) {
			obo_ref.setAttribute("name", "iodoacetamide");
			obo_ref.setAttribute("cvLabel", "PSI-MOD");
			obo_ref.setAttribute("accession", "MOD:00397");
			have_amodif = true;
			// iodoacetamide [MOD:00397] 57.021464
		}
		if (((F - 42.01) > -0.01) && ((F - 42.01) < 0.01)) {
			obo_ref.setAttribute("name", "N-acetylated residue");
			obo_ref.setAttribute("cvLabel", "PSI-MOD");
			obo_ref.setAttribute("accession", "MOD:00408");
			have_amodif = true;
			// N-acetylated residue [MOD:00408] 42.010565
		}
		if (((F - 79.966) > -0.01) && ((F - 79.966) < 0.01)) {
			obo_ref.setAttribute("name", "phosphorylated residue");
			obo_ref.setAttribute("cvLabel", "PSI-MOD");
			obo_ref.setAttribute("accession", "MOD:00696");
			have_amodif = true;
			// phosphorylated residue [MOD:00696] 79.966330
		}
		if (((F + 17.026) > -0.01) && ((F + 17.026) < 0.01)) {
			obo_ref.setAttribute("name", "loss of ammonia");
			obo_ref.setAttribute("cvLabel", "PSI-MOD");
			obo_ref.setAttribute("accession", "MOD:01160");
			have_amodif = true;
			// loss of ammonia [MOD:01160] -17.026549
		}
		if (((F + 18.01) > -0.01) && ((F + 18.01) < 0.01)) {
			obo_ref.setAttribute("name", "dehydrated residue");
			obo_ref.setAttribute("cvLabel", "PSI-MOD");
			obo_ref.setAttribute("accession", "MOD:00704");
			have_amodif = true;
			// dehydrated residue [MOD:00704] -18.010565
		}
		if (have_amodif == false) {
			obo_ref.setAttribute("name", "no");
		}
		return (obo_ref);
	}

	private void add_match() {
		Element matchs = doc.createElement("matchs");
		identificationrun.appendChild(matchs);
		for(Group group : ident.getGrouping().getGroupList()){
			for(SubGroup sg : group.getSubGroupSet().getSubGroupList()){
				Element match = doc.createElement("match");
				matchs.appendChild(match);
				
				// echantillons
				Hashtable<String, String> view = new Hashtable<String, String>();
				for(HashSampleScanSetProt hashProt : sg.getHashSampleScanSetProtList()){
					for(MsRun sample :hashProt.getMatch().get_sample_matchs()){
						if (!view.containsKey(sample.getSampleName())) {
							Element matchSample = doc
									.createElement("matchSample");
							matchSample.setAttribute("sample_id", ""
									+ sample_to_id.get(sample.getSampleName()));
							String name = sample.getSampleName();
							name = name.replaceAll(".xml", "");
							matchSample.setAttribute("name", "" + name);
							match.appendChild(matchSample);
							view.put(sample.getSampleName(), "ok");
						}
					}
				}

				// proteins
				int k =1;
				for(HashSampleScanSetProt hashProt : sg.getHashSampleScanSetProtList()){
					Element proteinHit = doc.createElement("proteinHit");
					Protein prot = hashProt.getMatch().get_protein_match();
					proteinHit.setAttribute("sequence_id",
							"" + prot_to_id.get(prot.get_description()));
					// proteinHit.setAttribute("score", "");
					proteinHit.setAttribute("rank", "" + k);
					match.appendChild(proteinHit);

					// //cvparam
					Element cvParam2 = doc.createElement("cvParam");
					cvParam2.setAttribute("value", "" + prot.get_mw());
					cvParam2.setAttribute("name", "MW determination by Benuch");
					cvParam2.setAttribute("cvLabel", "PROTICdbO");
					cvParam2.setAttribute("accession", "PROTICdbO:0000284");
					proteinHit.appendChild(cvParam2);

					// evalue
					MsRunSet samples = hashProt.getMatch().get_sample_matchs();
					Element cvParam1 = doc.createElement("cvParam");
					cvParam1.setAttribute("value", "" + prot.get_evalue());

					cvParam1.setAttribute("name", "Xtandem log evalue");
					cvParam1.setAttribute("cvLabel", "PROTICdbO");
					cvParam1.setAttribute("accession", "PROTICdbO:0000291");
					proteinHit.appendChild(cvParam1);

					// coverage
					if (samples.size() == 1) {
						Element cvParam3 = doc.createElement("cvParam");

						cvParam3.setAttribute(
								"value",
								""
										+  hashProt.getMatch().get_coverage());
						cvParam3.setAttribute("name",
								"Coverage determination by Benuch");
						cvParam3.setAttribute("cvLabel", "PROTICdbO");
						cvParam3.setAttribute("accession", "PROTICdbO:0000285");
						proteinHit.appendChild(cvParam3);
					}
					k++;
				}
				for(HashSampleScanSetProt hashProt : sg.getHashSampleScanSetProtList()){
					Protein prot2 =hashProt.getMatch().get_protein_match();
					// peptidesHitRef
					for(Peptide pep : hashProt.getMatch().get_peptide_order_inclus_redon()){
						Element peptideHitRef = doc
								.createElement("peptideHitRef");
						peptideHitRef
								.setAttribute(
										"peptide_hit_id",
										""
												+ pep_to_id.get(pep
														.get_sample_scan_sequencepeptide_cle()));
						match.appendChild(peptideHitRef);

						Element fromSeq = doc.createElement("fromSeq");
						fromSeq.setAttribute("seq_id",
								"" + prot_to_id.get(prot2.get_description()));
						fromSeq.setAttribute("start", "" + pep.get_start());
						fromSeq.setAttribute("stop", "" + pep.get_stop());
						String res1 = pep.get_pre();
						res1 = res1.replaceAll("]", "-");
						fromSeq.setAttribute("residue_before_nter",
								"" + res1.charAt((res1.length() - 1)));
						String res2 = pep.get_post();
						res2 = res2.replaceAll("]", "-");
						fromSeq.setAttribute("residue_after_cter",
								"" + res2.charAt(0));

						peptideHitRef.appendChild(fromSeq);
					}

				}
			}
		}
	}

	private void write() throws Exception {
		// écriture d'un document
		try {
			TransformerFactory tfact = TransformerFactory.newInstance();
			Transformer transformer = tfact.newTransformer();
			transformer.setOutputProperty("encoding", "UTF-8");
			transformer.setOutputProperty(OutputKeys.INDENT, "yes");
			DOMSource source = new DOMSource(this.doc);
			PrintWriter fw = new PrintWriter(this.proticfile, "UTF-8");
			StreamResult result = new StreamResult(fw);
			transformer.transform(source, result);
		} catch (Exception e) {
			throw new MSMSException("Impossible to save/write file : "
					+ this.proticfile.getName());
		}
	}

}
