package fr.inra.pappso.xtandempipeline.filter_print;

import java.util.ArrayList;
import java.util.TreeSet;

import fr.inra.pappso.libodsstream.OdsDocWriter;
import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.class_msms.Modifs;
import fr.inra.pappso.xtandempipeline.class_msms.Peptide;
import fr.inra.pappso.xtandempipeline.grouping.Group;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScan;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetPhospho;
import fr.inra.pappso.xtandempipeline.grouping.SubGroup;
import fr.inra.pappso.xtandempipeline.grouping.SubGroupSet;

public class print_ods_peptide_list_to_phosphopeptide extends printOdsBase {

	public print_ods_peptide_list_to_phosphopeptide(OdsDocWriter odsTable) {
		super(odsTable);
		logger.info("Print Peptide liste file in phospho mode ODS");
	}

	public void print_header() throws Exception {

		odsTable.writeLine();
		// out.write(samp.getSampleName() + "\n");
		ArrayList<String> headStrList = new ArrayList<String>();
		headStrList.add("Group");
		headStrList.add("Description");
		headStrList.add("Sample");
		headStrList.add("Scan");
		headStrList.add("Rt");
		headStrList.add("Sequence");
		headStrList.add("Best Position");
		headStrList.add("Phosphopeptide");
		headStrList.add("All Position");
		headStrList.add("Modifs");
		headStrList.add("Used");
		headStrList.add("on a total of");
		headStrList.add("Sub-groups");
		headStrList.add("E-value");
		headStrList.add("Charge");
		headStrList.add("MH+ Obs");
		headStrList.add("MH+ theo");
		headStrList.add("DeltaMH+");
		headStrList.add("Delta-ppm");

		for (String title : headStrList) {
			odsTable.writeCell(title);
		}
	}

	public void print_group_information(Group group) throws Exception {
		for (HashSampleScan hash : group.getHashSampleScanSet()
				.getHashSampleScanList()) {
			odsTable.writeLine();

			SubGroupSet set = group.getSubGroupContaining(hash);
			ArrayList<SubGroup> sgList = set.getSubGroupList();
			HashSampleScanSetPhospho phospho = (HashSampleScanSetPhospho) sgList
					.get(0).getHashSampleScanSetProtList().get(0);
			Match top = phospho.getMatch();
			Peptide pep = top.getBestPeptideToHashSampleScan(hash);
			// StringBuilder S = new StringBuilder();

			// on rempli
			odsTable.writeCell("" + group.getNumber());
			// S.append(group.getNumber() + "\t");
			if (set.size() == 1) {
				odsTable.writeCell(""
						+ top.get_protein_match().get_description());
				// S.append(top.get_protein_match().get_description());
			} else {
				odsTable.writeCell("");

			}
			odsTable.writeCell(""
					+ hash.getMsRun().getSampleName());
			// S.append("\t" + hash.getMsRun().getSampleName());
			odsTable.writeCell(new Double(hash.getScan()));
			// S.append("\t" + hash.getScan());
			odsTable.writeCell(new Double(pep.get_RT()));
			// S.append("\t" + pep.get_RT());
			odsTable.writeCell(pep.get_sequence());
			// S.append("\t" + pep.get_sequence());
			// position
			odsTable.writeCell(pep
					.get_phospho_Modifs_position());
			// S.append("\t" + pep.get_phospho_Modifs_position());

			// Comparaison des positions lié à l'incertitude
			TreeSet<Integer> posi = phospho.getAllPositionIncluderedon(hash);
			ArrayList<Modifs> phosphoModifs = pep.get_phospho_Modifs();

			// Multi
			if (phosphoModifs.size() >= 1) {
				if (phosphoModifs.size() > 1) {
					odsTable.writeCell("Multi");
					// S.append("\t" + "Multi" + "\t");
				} else {
					odsTable.writeCell("Mono");
					// S.append("\t" + "Mono" + "\t");
				}
				// S.append("\"");
				StringBuilder S = new StringBuilder();
				for (Integer pos : posi) {
					S.append(pos + " ");
				}
				odsTable.writeCell(S.toString());
				// S.append("\"");
			} else {
				odsTable.writeCell("No");
				odsTable.writeCell("-");
				// S.append("\t" + "No" + "\t" + "\"-\"");
			}

			odsTable.writeCell(pep.get_Modifs_formater());
			// S.append("\t" + pep.get_Modifs_formater());

			if (set.size() > 1) {
				odsTable.writeCell(new Double(set.size()));
						//S.append("\t" + set.size() + "\t"
				odsTable.writeCell(new Double(group.getSubGroupSet().size()));
					//	+ group.getSubGroupSet().size());
			} else {
				odsTable.writeCell("-");
				odsTable.writeCell("-");
				// S.append("\t" + "\"-\"" + "\t" + "\"-\"");
			}
			// S.append("\t");

			boolean second = false;
			StringBuilder S = new StringBuilder();
			for (SubGroup sub : sgList) {
				if (second) {
					S.append(" ");
				}
				S.append(this.get_subgroup_number(group, sub));
				second = true;
			}
			odsTable.writeCell(S.toString());

			odsTable.writeCell(new Double(pep.get_evalue()));
			// S.append("\t" + pep.get_evalue());
			odsTable.writeCell(new Double(pep.get_charge()));
			// S.append("\t" + pep.get_charge());
			odsTable.writeCell(new Double(pep.get_mhplus_obser()));
			// S.append("\t" + pep.get_mhplus_obser());
			odsTable.writeCell(new Double(pep.get_mhplus_theo()));
			S.append("\t" + pep.get_mhplus_theo());
			odsTable.writeCell(new Double(pep.get_deltamass()));
			// S.append("\t" + pep.get_deltamass());
			odsTable.writeCell(new Double(((pep.get_deltamass() / pep
					.get_mhplus_theo()) * 1000000)));
			// S.append("\t"
			// + ((pep.get_deltamass() / pep.get_mhplus_theo()) * 1000000));
			// S.append("\n");
			// out.write(S.toString());
		}
		odsTable.writeLine();
		// out.write("\n");
	}

}
